import geometry;

void distance(picture pic=currentpicture, Label L="", pair A, pair B,
              bool rotated=true, real offset=3mm,
              pen p=currentpen, pen joinpen=invisible,
              arrowbar arrow=Arrows(NoFill))
{
  pair A=A, B=B;
  guide g=A--B;
  transform T=shift(-offset*unit(B-A)*I);
  pic.add(new void(frame f, transform t) {
      picture opic;
      guide G=T*t*g;
      transform T=rotated ? rotate(B-A) : identity();
      draw(opic,T*Label(L,Center,UnFill(1)),G,p,arrow,Bars,PenMargins);
      pair Ap=t*A, Bp=t*B;
      draw(opic,(Ap--T*Ap)^^(Bp--T*Bp), joinpen);
      add(f,opic.fit());
    });
  pic.addBox(min(g),max(g),T*min(p),T*max(p));
}

picture fig1,fig2,fig3,fig4,fig5,fig6;
real xsize=10cm;
size(18cm,0);

pair A=(0,0),B=(1,0), C=(1,1), D=(0,1);
real x=1/3;
pair M=relpoint(A--B,x);
dot("$M$",M,SE);
draw(A--B--C--D--cycle);
label("$A$",A,SW); label("$B$",B,SE);
label("$C$",C,NE); label("$D$",D,NW);
distance("$x$",A,M);

add(fig1,currentpicture);
add(fig2,currentpicture);
add(fig3,currentpicture);
add(fig4,currentpicture);
add(fig5,currentpicture);
add(fig6,currentpicture);
erase();
// FIGURE 1

filldraw(box(A,(M.x,M.x)),grey);
filldraw(box((M.x,M.x),C),grey);
perpendicular((A.x,M.x),dir(45));
perpendicular((M.x,C.y),dir(225));
label("Figure $1$",truepoint(S),8*S);
add(fig1,currentpicture);
erase();

// FIGURE 2
filldraw(A--M--(A.x,M.x)--cycle,grey);
filldraw(box(M,C),grey);
distance("$x$",A,(A.x,M.x),false,-3mm);
perpendicular((M.x,C.y),dir(225));
label("Figure $2$",truepoint(S),8*S);
add(fig2,currentpicture);
erase();

// FIGURE 3
filldraw(box(A,(M.x,M.x)),grey);
draw(A--C,dotted);
perpendicular((A.x,M.x),dir(45));
perpendicular(M,dir(45));
label("Figure $3$",truepoint(S),8*S);
add(fig3,currentpicture);
erase();

// FIGURE 4
filldraw(A--B--C--(M.x,C.y)--(M.x,0.5)--(A.x,0.5)--cycle,grey);
draw(M--(M.x,0.5),dotted);
perpendicular((A.x,0.5),dir(45));
perpendicular((M.x,C.y),dir(225));
distance("$\frac{1}{2}$",A,(A.x,0.5),false,-3mm);
label("Figure $4$",truepoint(S),8*S);
add(fig4,currentpicture);
erase();

// FIGURE 5
filldraw(box((M.x,M.x),C),grey);
draw(A--C,dotted);
draw(M--(M.x,M.x),dotted);
perpendicular((B.x,M.x),dir(225));
perpendicular((M.x,C.y),dir(225));
label("Figure $5$",truepoint(S),8*S);
add(fig5,currentpicture);
erase();

// FIGURE 6
filldraw(box((M.x,M.x),(1-M.x,1-M.x))^^box(A,C),evenodd+grey);
draw(A--C^^B--D^^M--(M.x,M.x),dotted);
label("Figure $6$",truepoint(S),8*S);
add(fig6,currentpicture);
erase();


// Placement des figures
transform Th=shift(1.25,0);
transform Tv=shift(0,-1.5);
add(fig1);
add(Th*fig2);
add(Th^2*fig3);
add(Tv*fig4);
add(Th*Tv*fig5);
add(Th^2*Tv*fig6);

shipout(format="pdf");
